@@ -2,8 +2,8 @@
 set -e
 
 PYTORCH_VERSION=${PYTORCH_VERSION:-""}
-PYTHON_VERSION="3.7"
-TWINE_VERSION="1.12.1"
+PYTHON_VERSION="3.9"
+TWINE_VERSION=">3,<4.0.0dev"
 CONDA_ENV="skorch-deploy"
 CONDA_ENV_YML="environment.yml"
 
@@ -17,6 +17,9 @@ if [ -z "$PYTORCH_VERSION" ]; then
 	exit 1
 fi
 
+# check if worktree is not dirty, see https://stackoverflow.com/a/5737794
+test -z "$(git status --porcelain --untracked-files=no)"
+
 conda update -q -y conda
 
 # Remove previous deploy environment
@@ -42,10 +45,15 @@ trap remove_env EXIT
 source activate $CONDA_ENV
 conda install -q -y "twine==${TWINE_VERSION}"
 conda install -c pytorch -y "pytorch==${PYTORCH_VERSION}"
-python setup.py install
+python -m pip install -r requirements-dev.txt
+python -m pip install .
 
 pytest -x
 
+# check if README can be rendered correctly on PyPI
+python -m pip install readme-renderer
+python -m readme_renderer README.rst > /dev/null
+
 python setup.py sdist bdist_wheel
 
 if [[ $1 == "live" ]]; then
