@@ -7,6 +7,7 @@ from contextlib import suppress
 from fnmatch import fnmatch
 from functools import partial
 from itertools import product
+from copy import deepcopy
 
 import numpy as np
 from skorch.callbacks import Callback
@@ -19,7 +20,8 @@ from skorch.utils import unfreeze_parameter
 
 
 __all__ = ['Checkpoint', 'EarlyStopping', 'ParamMapper', 'Freezer',
-           'Unfreezer', 'Initializer', 'LoadInitState', 'TrainEndCheckpoint']
+           'Unfreezer', 'Initializer', 'InputShapeSetter', 'LoadInitState',
+           'TrainEndCheckpoint']
 
 
 class Checkpoint(Callback):
@@ -34,6 +36,11 @@ class Checkpoint(Callback):
     callback that dynamically evaluates whether the model should
     be saved in this epoch.
 
+    As checkpointing is often used in conjunction with early stopping
+    there is a need to restore the state of the model to the best
+    checkpoint after training is done. The checkpoint callback will
+    do this for you if you wish.
+
     Some or all of the following can be saved:
 
       - model parameters (see ``f_params`` parameter);
@@ -123,6 +130,13 @@ class Checkpoint(Callback):
     dirname: str (default='')
       Directory where files are stored.
 
+    load_best: bool (default=False)
+      Load the best checkpoint automatically once training ended.
+      This can be particularly helpful in combination with early stopping
+      as it allows for scoring with the best model, even when early stopping
+      ended training a number of epochs later. Note that this will only
+      work when ``monitor != None``.
+
     event_name: str, (default='event_cp')
       Name of event to be placed in history when checkpoint is triggered.
       Pass ``None`` to disable placing events in history.
@@ -145,6 +159,7 @@ class Checkpoint(Callback):
             dirname='',
             event_name='event_cp',
             sink=noop,
+            load_best=False,
             **kwargs
     ):
         self.monitor = monitor
@@ -157,6 +172,7 @@ class Checkpoint(Callback):
         self.dirname = dirname
         self.event_name = event_name
         self.sink = sink
+        self.load_best = load_best
         self._check_kwargs(kwargs)
         vars(self).update(**kwargs)
         self._validate_filenames()
@@ -174,6 +190,12 @@ class Checkpoint(Callback):
             os.makedirs(self.dirname, exist_ok=True)
         return self
 
+    def on_train_end(self, net, **kwargs):
+        if not self.load_best or self.monitor is None:
+            return
+        self._sink("Loading best checkpoint after training.", net.verbose)
+        net.load_params(checkpoint=self)
+
     def on_epoch_end(self, net, **kwargs):
         if "{}_best".format(self.monitor) in net.history[-1]:
             warnings.warn(
@@ -189,10 +211,10 @@ class Checkpoint(Callback):
             try:
                 do_checkpoint = net.history[-1, self.monitor]
             except KeyError as e:
-                raise SkorchException(
-                    "Monitor value '{}' cannot be found in history. "
-                    "Make sure you have validation data if you use "
-                    "validation scores for checkpointing.".format(e.args[0]))
+                msg = (
+                    f"{e.args[0]} Make sure you have validation data if you use "
+                    "validation scores for checkpointing.")
+                raise SkorchException(msg)
 
         if self.event_name is not None:
             net.history.record(self.event_name, bool(do_checkpoint))
@@ -346,6 +368,13 @@ class EarlyStopping(Callback):
       sent to. By default, the output is printed to stdout, but the
       sink could also be a logger or :func:`~skorch.utils.noop`.
 
+    load_best: bool (default=False)
+      Whether to restore module weights from the epoch with the best value of
+      the monitored quantity. If False, the module weights obtained at the
+      last step of training are used. Note that only the module is restored.
+      Use the ``Checkpoint`` callback with the :attr:`~Checkpoint.load_best`
+      argument set to ``True`` if you need to restore the whole object.
+
     """
     def __init__(
             self,
@@ -355,6 +384,7 @@ class EarlyStopping(Callback):
             threshold_mode='rel',
             lower_is_better=True,
             sink=print,
+            load_best=False,
     ):
         self.monitor = monitor
         self.lower_is_better = lower_is_better
@@ -364,6 +394,13 @@ class EarlyStopping(Callback):
         self.misses_ = 0
         self.dynamic_threshold_ = None
         self.sink = sink
+        self.load_best = load_best
+
+    def __getstate__(self):
+        # Avoids to save the module_ weights twice when pickling
+        state = self.__dict__.copy()
+        state['best_model_weights_'] = None
+        return state
 
     # pylint: disable=arguments-differ
     def on_train_begin(self, net, **kwargs):
@@ -372,6 +409,8 @@ class EarlyStopping(Callback):
                              .format(self.threshold_mode))
         self.misses_ = 0
         self.dynamic_threshold_ = np.inf if self.lower_is_better else -np.inf
+        self.best_model_weights_ = None
+        self.best_epoch_ = 0
 
     def on_epoch_end(self, net, **kwargs):
         current_score = net.history[-1, self.monitor]
@@ -380,6 +419,9 @@ class EarlyStopping(Callback):
         else:
             self.misses_ = 0
             self.dynamic_threshold_ = self._calc_new_threshold(current_score)
+            self.best_epoch_ = net.history[-1, "epoch"]
+            if self.load_best:
+                self.best_model_weights_ = deepcopy(net.module_.state_dict())
         if self.misses_ == self.patience:
             if net.verbose:
                 self._sink("Stopping since {} has not improved in the last "
@@ -387,6 +429,16 @@ class EarlyStopping(Callback):
                            verbose=net.verbose)
             raise KeyboardInterrupt
 
+    def on_train_end(self, net, **kwargs):
+        if (
+            self.load_best and (self.best_epoch_ != net.history[-1, "epoch"])
+            and (self.best_model_weights_ is not None)
+        ):
+            net.module_.load_state_dict(self.best_model_weights_)
+            self._sink("Restoring best model from epoch {}.".format(
+                self.best_epoch_
+            ), verbose=net.verbose)
+
     def _is_score_improved(self, score):
         if self.lower_is_better:
             return score < self.dynamic_threshold_
@@ -512,7 +564,7 @@ class ParamMapper(Callback):
         return self
 
     def named_parameters(self, net):
-        return net.module_.named_parameters()
+        return net.get_all_learnable_params()
 
     def filter_parameters(self, patterns, params):
         pattern_fns = (
@@ -619,13 +671,16 @@ class LoadInitState(Callback):
                        X=None, y=None, **kwargs):
         if not self.did_load_:
             self.did_load_ = True
-            with suppress(Exception):
-                net.load_params(checkpoint=self.checkpoint)
+            with suppress(FileNotFoundError):
+                if isinstance(self.checkpoint, TrainEndCheckpoint):
+                    net.load_params(checkpoint=self.checkpoint.checkpoint_)
+                else:
+                    net.load_params(checkpoint=self.checkpoint)
 
 
 class TrainEndCheckpoint(Callback):
     """Saves the model parameters, optimizer state, and history at the end of
-    training. The default ``fn_prefix`` is 'train_end_'.
+    training. The default ``fn_prefix`` is ``'train_end_'``.
 
     Examples
     --------
@@ -643,7 +698,6 @@ class TrainEndCheckpoint(Callback):
 
     Parameters
     ----------
-
     f_params : file-like object, str, None (default='params.pt')
       File path to the file or file-like object where the model
       parameters should be saved. Pass ``None`` to disable saving
@@ -732,10 +786,82 @@ class TrainEndCheckpoint(Callback):
             **self._f_kwargs()
         )
         self.checkpoint_.initialize()
+        return self
 
     def on_train_end(self, net, **kwargs):
         self.checkpoint_.save_model(net)
         self.checkpoint_._sink("Final checkpoint triggered", net.verbose)
+        return self
 
-    def __getattr__(self, attr):
-        return getattr(self.checkpoint_, attr)
+
+class InputShapeSetter(Callback):
+    """Sets the input dimension of the PyTorch module to the input dimension
+    of the training data. By default the last dimension of X (``X.shape[-1]``)
+    will be used.
+
+    This can be of use when the shape of X is not known beforehand,
+    e.g. when using a skorch model within an sklearn pipeline and
+    grid-searching feature transformers, or using feature selection
+    methods.
+
+    Basic usage:
+
+    >>> class MyModule(torch.nn.Module):
+    ...     def __init__(self, input_dim=1):
+    ...         super().__init__()
+    ...         self.layer = torch.nn.Linear(input_dim, 3)
+    ... # ...
+    >>> X1 = np.zeros(100, 5)
+    >>> X2 = np.zeros(100, 3)
+    >>> y = np.zeros(100)
+    >>> net = NeuralNetClassifier(MyModule, callbacks=[InputShapeSetter()])
+    >>> net.fit(X1, y)  # self.module_.layer.in_features == 5
+    >>> net.fit(X2, y)  # self.module_.layer.in_features == 3
+
+    Parameters
+    ----------
+    param_name : str (default='input_dim')
+      The parameter name is the parameter your model uses to define the
+      input dimension in its ``__init__`` method.
+
+    input_dim_fn : callable, None (default=None)
+      In case your ``X`` value is more complex and deriving the input
+      dimension is not as easy as ``X.shape[-1]`` you can pass a callable
+      to this parameter which takes ``X`` and returns the input dimension.
+
+    module_name : str (default='module')
+      Only needs change when you are using more than one module in your
+      skorch model (e.g., in case of GANs).
+    """
+    def __init__(
+        self,
+        param_name='input_dim',
+        input_dim_fn=None,
+        module_name='module',
+    ):
+        self.module_name = module_name
+        self.param_name = param_name
+        self.input_dim_fn = input_dim_fn
+
+    def get_input_dim(self, X):
+        if self.input_dim_fn is not None:
+            return self.input_dim_fn(X)
+        if len(X.shape) < 2:
+            raise ValueError(
+                "Expected at least two-dimensional input data for X. "
+                "If your data is one-dimensional, please use the "
+                "`input_dim_fn` parameter to infer the correct "
+                "input shape."
+            )
+        return X.shape[-1]
+
+    def on_train_begin(self, net, X, y, **kwargs):
+        params = net.get_params()
+        input_dim = self.get_input_dim(X)
+        param_name = f'{self.module_name}__{self.param_name}'
+
+        if params.get(param_name, None) == input_dim:
+            return
+
+        kwargs = {param_name: input_dim}
+        net.set_params(**kwargs)
