@@ -2,6 +2,8 @@
 
 from functools import partial
 import os
+import tempfile
+import unittest.mock
 from unittest.mock import Mock
 from unittest.mock import call, patch
 
@@ -11,13 +13,17 @@ import torch
 from torch import nn
 
 from skorch.tests.conftest import neptune_installed
+from skorch.tests.conftest import sacred_installed
 from skorch.tests.conftest import wandb_installed
 from skorch.tests.conftest import tensorboard_installed
-
+from skorch.tests.conftest import mlflow_installed
 
 @pytest.mark.skipif(
     not neptune_installed, reason='neptune is not installed')
 class TestNeptune:
+    # fields logged by on_train_begin and on_train_end
+    NUM_BASE_METRICS = 9
+
     @pytest.fixture
     def net_cls(self):
         from skorch import NeuralNetClassifier
@@ -36,18 +42,24 @@ class TestNeptune:
         return NeptuneLogger
 
     @pytest.fixture
-    def neptune_experiment_cls(self):
-        import neptune
-        neptune.init(project_qualified_name="tests/dry-run",
-                     backend=neptune.OfflineBackend())
-        return neptune.create_experiment
+    def neptune_run_object(self):
+        try:
+            # neptune-client=0.9.0+ package structure
+            import neptune.new as neptune
+        except ImportError:
+            # neptune-client>=1.0.0 package structure
+            import neptune
+
+        run = neptune.init_run(
+            project="tests/dry-run",
+            mode="offline",
+        )
+        return run
 
     @pytest.fixture
-    def mock_experiment(self, neptune_experiment_cls):
-        mock = Mock(spec=neptune_experiment_cls)
-        mock.log_metric = Mock()
-        mock.stop = Mock()
-        return mock
+    def mock_experiment(self, neptune_run_object):
+        with neptune_run_object as run:
+            return unittest.mock.create_autospec(run)
 
     @pytest.fixture
     def net_fitted(
@@ -67,6 +79,10 @@ class TestNeptune:
     def test_experiment_closed_automatically(self, net_fitted, mock_experiment):
         assert mock_experiment.stop.call_count == 1
 
+    def test_experiment_log_call_counts(self, net_fitted, mock_experiment):
+        # (3 x dur + 3 x train_loss + 3 x valid_loss + 3 x valid_acc = 12) + base metrics
+        assert mock_experiment.__getitem__.call_count == 12 + self.NUM_BASE_METRICS
+
     def test_experiment_not_closed(
             self,
             net_cls,
@@ -101,10 +117,8 @@ class TestNeptune:
             max_epochs=3,
         ).fit(*data)
 
-        # 3 epochs x 2 epoch metrics = 6 calls
-        assert mock_experiment.log_metric.call_count == 6
-        call_args = [args[0][0] for args in mock_experiment.log_metric.call_args_list]
-        assert 'valid_loss' not in call_args
+        # (3 epochs x 2 epoch metrics = 6 calls) + base metrics
+        assert mock_experiment.__getitem__.call_count == 6 + self.NUM_BASE_METRICS
 
     def test_keys_ignored_is_string(self, neptune_logger_cls, mock_experiment):
         npt = neptune_logger_cls(
@@ -118,15 +132,23 @@ class TestNeptune:
             classifier_module,
             data,
             neptune_logger_cls,
-            neptune_experiment_cls,
+            neptune_run_object,
     ):
         net = net_cls(
             classifier_module,
-            callbacks=[neptune_logger_cls(neptune_experiment_cls())],
+            callbacks=[neptune_logger_cls(neptune_run_object)],
             max_epochs=5,
         )
         net.fit(*data)
 
+        assert neptune_run_object.exists('training/epoch_duration')
+        assert neptune_run_object.exists('training/train/epoch/loss')
+        assert neptune_run_object.exists('training/validation/epoch/loss')
+        assert neptune_run_object.exists('training/validation/epoch/acc')
+
+        # Checkpoint callback was not used
+        assert not neptune_run_object.exists('training/model/checkpoint')
+
     def test_log_on_batch_level_on(
             self,
             net_cls,
@@ -144,9 +166,9 @@ class TestNeptune:
         )
         net.fit(*data)
 
-        # 5 epochs x (40/4 batches x 2 batch metrics + 2 epoch metrics) = 110 calls
-        assert mock_experiment.log_metric.call_count == 110
-        mock_experiment.log_metric.assert_any_call('train_batch_size', 4)
+        # (5 epochs x (40/4 batches x 2 batch metrics + 2 epoch metrics) = 110 calls) + base metrics
+        assert mock_experiment.__getitem__.call_count == 110 + self.NUM_BASE_METRICS
+        mock_experiment['training']['train']['batch']['batch_size'].log.assert_any_call(4)
 
     def test_log_on_batch_level_off(
             self,
@@ -165,31 +187,204 @@ class TestNeptune:
         )
         net.fit(*data)
 
-        # 5 epochs x 2 epoch metrics = 10 calls
-        assert mock_experiment.log_metric.call_count == 10
-        call_args_list = mock_experiment.log_metric.call_args_list
-        assert call('train_batch_size', 4) not in call_args_list
+        # (5 epochs x 2 epoch metrics = 10 calls) + base metrics
+        assert mock_experiment.__getitem__.call_count == 10 + self.NUM_BASE_METRICS
 
-    def test_first_batch_flag(
+        call_args = mock_experiment['training']['train'].__getitem__.call_args_list
+        assert call('epoch') in call_args
+        assert call('batch') not in call_args
+
+        call_args = mock_experiment['training']['validation'].__getitem__.call_args_list
+        assert call('epoch') in call_args
+        assert call('batch') not in call_args
+
+    def test_fit_with_real_experiment_saving_checkpoints(
             self,
             net_cls,
             classifier_module,
             data,
             neptune_logger_cls,
-            neptune_experiment_cls,
+            neptune_run_object,
     ):
-        npt = neptune_logger_cls(neptune_experiment_cls())
-        npt.initialize()
-        assert npt.first_batch_ is True
+        try:
+            # neptune-client=0.9.0+ package structure
+            from neptune.new.attributes.file_set import FileSet
+        except ImportError:
+            # neptune-client>=1.0.0 package structure
+            from neptune.attributes.file_set import FileSet
+        from skorch.callbacks import Checkpoint
 
-        net = net_cls(
-            classifier_module,
-            callbacks=[npt],
-            max_epochs=1,
+        with tempfile.TemporaryDirectory() as directory:
+            net = net_cls(
+                classifier_module,
+                callbacks=[
+                    neptune_logger_cls(
+                        run=neptune_run_object,
+                        close_after_train=False,
+                    ),
+                    Checkpoint(dirname=directory),
+                ],
+                max_epochs=5,
+            )
+            net.fit(*data)
+            neptune_run_object['training/model/checkpoint'].upload_files(directory)
+
+        assert neptune_run_object.exists('training/train/epoch/loss')
+        assert neptune_run_object.exists('training/validation/epoch/loss')
+        assert neptune_run_object.exists('training/validation/epoch/acc')
+
+        assert neptune_run_object.exists('training/model/checkpoint')
+        assert isinstance(
+            neptune_run_object.get_structure()['training']['model']['checkpoint'],
+            FileSet,
         )
 
-        npt.on_batch_end(net)
-        assert npt.first_batch_ is False
+        neptune_run_object.stop()
+
+@pytest.mark.skipif(
+    not sacred_installed, reason='Sacred is not installed')
+class TestSacred:
+    @pytest.fixture
+    def net_cls(self):
+        from skorch import NeuralNetClassifier
+        return NeuralNetClassifier
+
+    @pytest.fixture
+    def data(self, classifier_data):
+        X, y = classifier_data
+        # accelerate training since we don't care for the loss
+        X, y = X[:40], y[:40]
+        return X, y
+
+    @pytest.fixture
+    def sacred_logger_cls(self):
+        from skorch.callbacks import SacredLogger
+        return SacredLogger
+
+    @pytest.fixture
+    def sacred_experiment_cls(self):
+        from sacred import Experiment
+        return Experiment
+
+    @pytest.fixture
+    def mock_experiment(self, sacred_experiment_cls):
+        mock = Mock(spec=sacred_experiment_cls)
+        mock.log_scalar = Mock()
+        return mock
+
+    @pytest.fixture
+    def net_fitted(
+            self,
+            net_cls,
+            classifier_module,
+            data,
+            sacred_logger_cls,
+            mock_experiment,
+    ):
+        return net_cls(
+            classifier_module,
+            callbacks=[sacred_logger_cls(mock_experiment)],
+            max_epochs=3,
+        ).fit(*data)
+
+    def test_ignore_keys(
+            self,
+            net_cls,
+            classifier_module,
+            data,
+            sacred_logger_cls,
+            mock_experiment,
+    ):
+        # ignore 'dur' and 'valid_loss', 'unknown' doesn't exist but
+        # this should not cause a problem
+        logger = sacred_logger_cls(
+            mock_experiment, keys_ignored=['dur', 'valid_loss', 'unknown'])
+        net_cls(
+            classifier_module,
+            callbacks=[logger],
+            max_epochs=3,
+        ).fit(*data)
+
+        # 3 epochs x 2 epoch metrics = 6 calls
+        assert mock_experiment.log_scalar.call_count == 6
+        call_args = [args[0][0] for args in mock_experiment.log_scalar.call_args_list]
+        assert 'valid_loss' not in call_args
+
+    def test_keys_ignored_is_string(self, sacred_logger_cls, mock_experiment):
+        logger = sacred_logger_cls(
+            mock_experiment, keys_ignored='a-key').initialize()
+        expected = {'a-key', 'batches'}
+        assert logger.keys_ignored_ == expected
+
+    def test_fit_with_real_experiment(
+            self,
+            net_cls,
+            classifier_module,
+            data,
+            sacred_logger_cls,
+            sacred_experiment_cls,
+    ):
+        experiment = sacred_experiment_cls()
+
+        @experiment.main
+        def experiment_main(_run):
+            net = net_cls(
+                classifier_module,
+                callbacks=[sacred_logger_cls(_run)],
+                max_epochs=5,
+            )
+            net.fit(*data)
+
+        experiment.run()
+
+    def test_log_on_batch_level_on(
+            self,
+            net_cls,
+            classifier_module,
+            data,
+            sacred_logger_cls,
+            mock_experiment,
+    ):
+        net = net_cls(
+            classifier_module,
+            callbacks=[sacred_logger_cls(mock_experiment, log_on_batch_end=True)],
+            max_epochs=5,
+            batch_size=4,
+            train_split=False
+        )
+        net.fit(*data)
+
+        # 5 epochs x (40/4 batches x 2 batch metrics + 2 epoch metrics) = 110 calls
+        assert mock_experiment.log_scalar.call_count == 110
+        mock_experiment.log_scalar.assert_any_call('train_batch_size_batch', 4)
+
+        logged_keys = [
+            call_args.args[0] for call_args in mock_experiment.log_scalar.call_args_list
+        ]
+        # This is a batch-only metric.
+        assert 'train_batch_size_epoch' not in logged_keys
+
+    def test_log_on_batch_level_off(
+            self,
+            net_cls,
+            classifier_module,
+            data,
+            sacred_logger_cls,
+            mock_experiment,
+    ):
+        net = net_cls(
+            classifier_module,
+            callbacks=[sacred_logger_cls(mock_experiment, log_on_batch_end=False)],
+            max_epochs=5,
+            batch_size=4,
+            train_split=False
+        )
+        net.fit(*data)
+
+        # 5 epochs x 2 epoch metrics = 10 calls
+        assert mock_experiment.log_scalar.call_count == 10
+        call_args_list = mock_experiment.log_scalar.call_args_list
+        assert call('train_batch_size_batch', 4) not in call_args_list
 
 @pytest.mark.skipif(
     not wandb_installed, reason='wandb is not installed')
@@ -215,8 +410,12 @@ class TestWandb:
     def wandb_run_cls(self):
         import wandb
         os.environ['WANDB_MODE'] = 'dryrun' # run offline
-        with wandb.init(anonymous="allow") as run:
-            return run
+        wandb_version = tuple(map(int, wandb.__version__.split('.')[:2]))
+        if wandb_version >= (0, 10):
+            return wandb.init(anonymous="allow")
+        else:
+            with wandb.init(anonymous="allow") as run:
+                return run
 
     @pytest.fixture
     def mock_run(self):
@@ -791,3 +990,185 @@ class TestTensorBoard:
 
         # is not empty
         assert os.listdir(path)
+
+
+@pytest.mark.skipif(
+    not mlflow_installed, reason='mlflow is not installed')
+class TestMLflowLogger:
+    @pytest.fixture
+    def net_cls(self):
+        from skorch import NeuralNetClassifier
+        return NeuralNetClassifier
+
+    @pytest.fixture
+    def logger_cls(self):
+        from skorch.callbacks import MlflowLogger
+        return MlflowLogger
+
+    @pytest.fixture
+    def data(self, classifier_data):
+        X, y = classifier_data
+        X, y = X[:40], y[:40]
+        return X, y
+
+    @pytest.fixture
+    def mock_run(self):
+        from mlflow.entities import Run
+        return Mock(Run)
+
+    @pytest.fixture
+    def mock_client(self):
+        from mlflow.tracking import MlflowClient
+        return Mock(MlflowClient)
+
+    @pytest.fixture
+    def logger_mock_cls(self, logger_cls, mock_run, mock_client):
+        return partial(logger_cls, mock_run, mock_client)
+
+    @pytest.fixture
+    def net_builder_cls(self, net_cls, classifier_module, data):
+        def builder(*args, **kwargs):
+            return net_cls(classifier_module, *args, **kwargs).fit(*data)
+        return builder
+
+    @pytest.fixture
+    def net_fitted(self, logger_mock_cls, net_builder_cls):
+        return net_builder_cls(callbacks=[logger_mock_cls()], max_epochs=3)
+
+    def test_run_default(self, monkeypatch, logger_cls, mock_run, mock_client):
+        import mlflow
+        mock_active_run = Mock(mlflow.active_run, return_value=mock_run)
+        monkeypatch.setattr(mlflow, 'active_run', mock_active_run)
+        logger = logger_cls(client=mock_client).initialize()
+        assert mock_active_run.called
+        assert logger.run_ == mock_run
+
+    def test_client_default(self, monkeypatch, logger_cls, mock_run, mock_client):
+        import mlflow.tracking
+        monkeypatch.setattr(mlflow.tracking, 'MlflowClient', mock_client)
+        logger = logger_cls(run=mock_run).initialize()
+        assert mock_client.called
+        assert logger.client_ == mock_client()
+
+    def test_keys_from_history_logged(self, net_fitted, mock_client):
+        assert mock_client.log_metric.call_count == 3 * 4
+        keys = {call_args[0][1] for call_args in mock_client.log_metric.call_args_list}
+        expected = {'dur', 'train_loss', 'valid_loss', 'valid_acc'}
+        assert keys == expected
+
+    def test_ignore_keys(self, logger_mock_cls, net_builder_cls):
+        # ignore 'dur' and 'valid_loss', 'unknown' doesn't exist but
+        # this should not cause a problem
+        logger = logger_mock_cls(keys_ignored=['dur', 'valid_loss', 'unknown'])
+        net_builder_cls(callbacks=[logger], max_epochs=3)
+        keys = {
+            call_args[0][1]
+            for call_args in logger.client_.log_metric.call_args_list
+        }
+        expected = {'train_loss', 'valid_acc'}
+        assert keys == expected
+
+    def test_keys_ignored_is_string(self, logger_mock_cls):
+        logger = logger_mock_cls(keys_ignored='a-key').initialize()
+        expected = {'a-key', 'batches'}
+        assert logger.keys_ignored_ == expected
+
+    @pytest.mark.parametrize(
+        'log_on_batch_end, log_on_epoch_end, batch_suffix, epoch_suffix',
+        [(False, False, '', ''),
+         (True, False, '', ''),
+         (False, True, '', ''),
+         (True, True, '_batch', '_epoch')]
+    )
+    def test_epoch_batch_suffixes_defaults(
+            self,
+            logger_mock_cls,
+            log_on_batch_end,
+            log_on_epoch_end,
+            batch_suffix,
+            epoch_suffix,
+    ):
+        logger = logger_mock_cls(
+            log_on_batch_end=log_on_batch_end,
+            log_on_epoch_end=log_on_epoch_end
+        ).initialize()
+        assert logger.batch_suffix_ == batch_suffix
+        assert logger.epoch_suffix_ == epoch_suffix
+
+    @pytest.mark.parametrize('batch_suffix', ['', '_foo'])
+    @pytest.mark.parametrize('epoch_suffix', ['', '_bar'])
+    def test_epoch_batch_custom_suffix(
+            self,
+            logger_mock_cls,
+            batch_suffix,
+            epoch_suffix,
+    ):
+        logger = logger_mock_cls(
+            log_on_batch_end=True,
+            log_on_epoch_end=True,
+            batch_suffix=batch_suffix,
+            epoch_suffix=epoch_suffix,
+        ).initialize()
+        assert logger.batch_suffix_ == batch_suffix
+        assert logger.epoch_suffix_ == epoch_suffix
+
+    def test_dont_log_epoch_metrics(self, logger_mock_cls, net_builder_cls):
+        logger = logger_mock_cls(
+            log_on_batch_end=True,
+            log_on_epoch_end=False,
+            batch_suffix='_batch',
+            epoch_suffix='_epoch',
+        )
+        net_builder_cls(batch_size=10, callbacks=[logger], max_epochs=3)
+        assert all(
+            call[0][1].endswith('_batch')
+            for call in logger.client_.log_metric.call_args_list
+        )
+
+    def test_log_epochs_with_step(self, net_fitted, mock_client):
+        expected = [x for x in range(1, 4) for _ in range(4)]
+        actual = [call[1].get('step') for call in mock_client.log_metric.call_args_list]
+        assert expected == actual
+
+    def test_log_batch_with_step(self, logger_mock_cls, net_builder_cls):
+        logger = logger_mock_cls(log_on_batch_end=True, log_on_epoch_end=False)
+        net_builder_cls(batch_size=10, callbacks=[logger], max_epochs=4)
+        expected = [x for x in range(1, 21) for _ in range(2)]
+        actual = [
+            call[1].get('step')
+            for call in logger.client_.log_metric.call_args_list
+        ]
+        assert expected == actual
+
+    def test_artifact_filenames(self, net_fitted, mock_client):
+        keys = {call_args[0][1].name
+                for call_args in mock_client.log_artifact.call_args_list}
+        expected = {'params.pth', 'optimizer.pth', 'criterion.pth', 'history.json'}
+        assert keys == expected
+
+    def test_artifact_in_temporary_directory(self, net_fitted, mock_client):
+        for call_args in mock_client.log_artifact.call_args_list:
+            assert str(call_args[0][1]).startswith('/tmp')
+
+    def test_dont_create_artifact(self, net_builder_cls, logger_mock_cls):
+        logger = logger_mock_cls(create_artifact=False)
+        net_builder_cls(callbacks=[logger], max_epochs=3)
+        assert not logger.client_.log_artifact.called
+
+    def test_run_terminated_automatically(self, net_fitted, mock_client):
+        assert mock_client.set_terminated.call_count == 1
+
+    def test_run_not_closed(self, logger_mock_cls, mock_client, net_builder_cls):
+        logger = logger_mock_cls(terminate_after_train=False)
+        net_builder_cls(callbacks=[logger], max_epochs=2)
+        assert logger.client_.set_terminated.call_count == 0
+
+    def test_fit_with_real_run_and_client(self, tmp_path, logger_cls, net_builder_cls):
+        from mlflow.tracking import MlflowClient
+        client = MlflowClient(tracking_uri=tmp_path.as_uri())
+        experiment_name = 'foo'
+        experiment_id = client.create_experiment(experiment_name)
+        run = client.create_run(experiment_id)
+        logger = logger_cls(run, client, create_artifact=False)
+        net_builder_cls(callbacks=[logger], max_epochs=3)
+        assert os.listdir(tmp_path)
