@@ -330,7 +330,6 @@ class TestEpochScoring:
         def myscore(net, X, y=None):
             nonlocal score_calls
             score_calls += 1
-            assert y is None
 
             # In case we use caching X is a dataset. We need to
             # extract X ourselves.
@@ -388,7 +387,7 @@ class TestEpochScoring:
         # received at the scoring side as well. For the caching case
         # we only receive datasets.
         import skorch
-        from skorch.dataset import CVSplit
+        from skorch.dataset import ValidSplit
         import torch.utils.data.dataset
         from torch.utils.data.dataset import Subset
 
@@ -402,7 +401,7 @@ class TestEpochScoring:
             pass
 
         rawsplit = lambda ds: (ds, ds)
-        cvsplit = CVSplit(2, random_state=0)
+        valid_split = ValidSplit(2)
 
         def split_ignore_y(ds, y):
             return rawsplit(ds)
@@ -417,12 +416,12 @@ class TestEpochScoring:
             ((MySkorchDataset(*data), None), rawsplit, MySkorchDataset, True),
 
             # Test a split that splits datasets using torch Subset
-            (data, cvsplit, np.ndarray, False),
-            (data, cvsplit, Subset, True),
-            ((MyTorchDataset(*data), None), cvsplit, Subset, False),
-            ((MyTorchDataset(*data), None), cvsplit, Subset, True),
-            ((MySkorchDataset(*data), None), cvsplit, np.ndarray, False),
-            ((MySkorchDataset(*data), None), cvsplit, Subset, True),
+            (data, valid_split, np.ndarray, False),
+            (data, valid_split, Subset, True),
+            ((MyTorchDataset(*data), None), valid_split, Subset, False),
+            ((MyTorchDataset(*data), None), valid_split, Subset, True),
+            ((MySkorchDataset(*data), None), valid_split, np.ndarray, False),
+            ((MySkorchDataset(*data), None), valid_split, Subset, True),
         ]
 
         for input_data, train_split, expected_type, caching in table:
@@ -480,7 +479,7 @@ class TestEpochScoring:
         with pytest.raises(ValueError, match=msg):
                 net.fit(*data)
 
-    @pytest.mark.parametrize('use_caching, count', [(False, 1), (True, 0)])
+    @pytest.mark.parametrize('use_caching, count', [(False, 5), (True, 2)])
     def test_with_caching_get_iterator_not_called(
             self, net_cls, module_cls, train_split, caching_scoring_cls, data,
             use_caching, count,
@@ -500,10 +499,11 @@ class TestEpochScoring:
         net.fit(*data)
 
         # expected count should be:
-        # max_epochs * (1 (train) + 1 (valid) + 0 or 1 (from scoring,
-        # depending on caching))
-        count_expected = max_epochs * (1 + 1 + count)
-        assert net.get_iterator.call_count == count_expected
+        # fit loop: 1 (train) + 1 (valid) = 2
+        # scoring:
+        #   without cahching: 0
+        #   with caching: 1 per epoch = 3
+        assert net.get_iterator.call_count == count
 
     def test_subclassing_epoch_scoring(
             self, classifier_module, classifier_data):
@@ -894,7 +894,6 @@ class TestBatchScoring:
         def myscore(net, X, y=None):  # pylint: disable=unused-argument
             nonlocal score_calls
             score_calls += 1
-            assert y is None
             return X.mean().data.item()
 
         # pylint: disable=unused-argument
