@@ -1,13 +1,21 @@
 """Tests for callbacks in training.py"""
 
 from functools import partial
+import pickle
 from unittest.mock import Mock
 from unittest.mock import patch
 from unittest.mock import call
+from copy import deepcopy
 
 import numpy as np
 import pytest
 from sklearn.base import clone
+from sklearn.model_selection import train_test_split
+from sklearn.metrics import log_loss
+import torch
+from torch.utils.data import TensorDataset
+
+from skorch.helper import predefined_split
 
 
 class TestCheckpoint:
@@ -177,15 +185,14 @@ class TestCheckpoint:
             train_split=None
         )
         from skorch.exceptions import SkorchException
-        with pytest.raises(SkorchException) as e:
+
+        msg_expected = (
+            r"Key 'valid_loss_best' was not found in history. "
+            r"Make sure you have validation data if you use "
+            r"validation scores for checkpointing."
+        )
+        with pytest.raises(SkorchException, match=msg_expected):
             net.fit(*data)
-            expected = (
-                "Monitor value '{}' cannot be found in history. "
-                "Make sure you have validation data if you use "
-                "validation scores for checkpointing.".format(
-                    'valid_loss_best')
-            )
-            assert str(e.value) == expected
 
     def test_string_monitor_and_formatting(
             self, save_params_mock, net_cls, checkpoint_cls, data):
@@ -320,7 +327,7 @@ class TestCheckpoint:
         assert pickle_dump_mock.call_count == len(net.history)
         save_params_mock.assert_has_calls(
             [
-                call(f_module=str(f_params)),  # params is turned into module 
+                call(f_module=str(f_params)),  # params is turned into module
                 call(f_optimizer=str(f_optimizer)),
                 call(f_criterion=str(f_criterion)),
                 call(f_history=str(f_history)),
@@ -398,6 +405,48 @@ class TestCheckpoint:
             [call(f_mymodule='mymodule.pt')] * len(net.history)
         )
 
+    @pytest.fixture
+    def load_params(self):
+        import torch
+        return torch.load
+
+    @pytest.mark.parametrize('load_best_flag', [False, True])
+    def test_automatically_load_checkpoint(
+            self, net_cls, checkpoint_cls, data, tmp_path,
+            load_params, load_best_flag,
+    ):
+        # checkpoint once at the beginning of training.
+        # when restoring at the end of training, the parameters
+        # of the net should not differ. If we do not restore
+        # then the parameters must differ.
+        path_cb = tmp_path / 'params_cb.pt'
+        path_net = tmp_path / 'params_net.pt'
+
+        def save_once_monitor(net):
+            return len(net.history) == 1
+
+        net = net_cls(
+            max_epochs=3,
+            callbacks=[
+                checkpoint_cls(
+                    monitor=save_once_monitor,
+                    f_params=path_cb,
+                    load_best=load_best_flag,
+                ),
+            ],
+        )
+
+        net.fit(*data)
+        net.save_params(path_net)
+
+        params_cb = load_params(path_cb)
+        params_net = load_params(path_net)
+
+        if load_best_flag:
+            assert params_cb == params_net
+        else:
+            assert params_cb != params_net
+
 
 
 class TestEarlyStopping:
@@ -443,6 +492,94 @@ class TestEarlyStopping:
 
         assert len(net.history) == max_epochs
 
+    def test_weights_restore(
+            self, net_clf_cls, classifier_module, classifier_data,
+            early_stopping_cls):
+        patience = 3
+        max_epochs = 20
+        seed = 1
+
+        side_effect = []
+
+        def sink(x):
+            side_effect.append(x)
+
+        early_stopping_cb = early_stopping_cls(
+            patience=patience,
+            sink=sink,
+            load_best=True,
+            monitor="valid_acc",
+            lower_is_better=False,
+        )
+
+        # Split dataset to have a fixed validation
+        X_tr, X_val, y_tr, y_val = train_test_split(
+            *classifier_data, random_state=seed)
+        tr_dataset = TensorDataset(
+            torch.as_tensor(X_tr).float(), torch.as_tensor(y_tr))
+        val_dataset = TensorDataset(
+            torch.as_tensor(X_val).float(), torch.as_tensor(y_val))
+
+        # Fix the network once with early stoppping and fixed seed
+        net1 = net_clf_cls(
+            classifier_module,
+            callbacks=[early_stopping_cb],
+            max_epochs=max_epochs,
+            train_split=predefined_split(val_dataset),
+        )
+        torch.manual_seed(seed)
+        net1.fit(tr_dataset, y=None)
+
+        # Check training was stopped before the end
+        assert len(net1.history) < max_epochs
+
+        # check correct output messages
+        assert len(side_effect) == 2
+
+        msg = side_effect[0]
+        expected_msg = ("Stopping since valid_acc has not improved in "
+                        "the last 3 epochs.")
+        assert msg == expected_msg
+
+        msg = side_effect[1]
+        expected_msg = "Restoring best model from epoch "
+        assert expected_msg in msg
+
+        # Recompute validation loss and store it together with module weights
+        y_proba = net1.predict_proba(val_dataset)
+        es_weights = deepcopy(net1.module_.state_dict())
+        es_loss = log_loss(y_val, y_proba)
+
+        # Retrain same classifier without ES, using the best epochs number
+        net2 = net_clf_cls(
+            classifier_module,
+            max_epochs=early_stopping_cb.best_epoch_,
+            train_split=predefined_split(val_dataset),
+        )
+        torch.manual_seed(seed)
+        net2.fit(tr_dataset, y=None)
+
+        # Check that weights obtained match
+        assert all(
+            torch.equal(wi, wj)
+            for wi, wj in zip(
+                net2.module_.state_dict().values(),
+                es_weights.values()
+            )
+        )
+
+        # Check validation loss obtained match
+        y_proba_2 = net2.predict_proba(val_dataset)
+        assert es_loss == log_loss(y_val, y_proba_2)
+
+        # Check best_model_weights_ is transformed into None when pickling
+        del net1.callbacks[0].sink
+        net1_pkl = pickle.dumps(net1)
+
+        reloaded_net1 = pickle.loads(net1_pkl)
+        assert reloaded_net1.callbacks[0].best_epoch_ == net1.callbacks[0].best_epoch_
+        assert reloaded_net1.callbacks[0].best_model_weights_ is None
+
     def test_typical_use_case_stopping(
             self, net_clf_cls, broken_classifier_module, classifier_data,
             early_stopping_cls):
@@ -1083,3 +1220,198 @@ class TestTrainEndCheckpoint:
 
         assert save_params_mock.call_count == 1
         save_params_mock.assert_has_calls([call(f_mymodule='train_end_mymodule.pt')])
+
+    def test_pickle_uninitialized_callback(self, trainendcheckpoint_cls):
+        # isuue 773
+        cp = trainendcheckpoint_cls()
+        # does not raise
+        s = pickle.dumps(cp)
+        pickle.loads(s)
+
+    def test_pickle_initialized_callback(self, trainendcheckpoint_cls):
+        # issue 773
+        cp = trainendcheckpoint_cls().initialize()
+        # does not raise
+        s = pickle.dumps(cp)
+        pickle.loads(s)
+
+
+class TestInputShapeSetter:
+
+    @pytest.fixture
+    def module_cls(self):
+        import torch
+
+        class Module(torch.nn.Module):
+            def __init__(self, input_dim=3):
+                super().__init__()
+                self.layer = torch.nn.Linear(input_dim, 2)
+            def forward(self, X):
+                return self.layer(X)
+
+        return Module
+
+    @pytest.fixture
+    def net_cls(self):
+        from skorch import NeuralNetClassifier
+        return NeuralNetClassifier
+
+    @pytest.fixture
+    def input_shape_setter_cls(self):
+        from skorch.callbacks import InputShapeSetter
+        return InputShapeSetter
+
+    def generate_data(self, n_input):
+        from sklearn.datasets import make_classification
+        X, y = make_classification(
+            1000,
+            n_input,
+            n_informative=n_input,
+            n_redundant=0,
+            random_state=0,
+        )
+        return X.astype(np.float32), y
+
+    @pytest.fixture
+    def data_fixed(self):
+        return self.generate_data(n_input=10)
+
+    @pytest.fixture(params=[2, 10, 20])
+    def data_parametrized(self, request):
+        return self.generate_data(n_input=request.param)
+
+    def test_shape_set(
+        self, net_cls, module_cls, input_shape_setter_cls, data_parametrized,
+    ):
+        net = net_cls(module_cls, max_epochs=2, callbacks=[
+            input_shape_setter_cls(),
+        ])
+
+        X, y = data_parametrized
+        n_input = X.shape[1]
+        net.fit(X, y)
+
+        assert net.module_.layer.in_features == n_input
+
+    def test_one_dimensional_x_raises(
+        self, net_cls, module_cls, input_shape_setter_cls,
+    ):
+        net = net_cls(module_cls, max_epochs=2, callbacks=[
+            input_shape_setter_cls(),
+        ])
+
+        X, y = np.zeros(10), np.zeros(10)
+
+        with pytest.raises(ValueError) as e:
+            net.fit(X, y)
+
+        assert (
+            "Expected at least two-dimensional input data for X. "
+            "If your data is one-dimensional, please use the `input_dim_fn` "
+            "parameter to infer the correct input shape."
+            ) in str(e)
+
+    def test_shape_set_using_fn(
+        self, net_cls, module_cls, input_shape_setter_cls, data_parametrized,
+    ):
+        fn_calls = 0
+
+        def input_dim_fn(X):
+            nonlocal fn_calls
+            fn_calls += 1
+            return X.shape[1]
+
+        net = net_cls(module_cls, max_epochs=2, callbacks=[
+            input_shape_setter_cls(input_dim_fn=input_dim_fn),
+        ])
+
+        X, y = data_parametrized
+        n_input = X.shape[1]
+        net.fit(X, y)
+
+        assert net.module_.layer.in_features == n_input
+        assert fn_calls == 1
+
+    def test_parameter_name(
+        self, net_cls, input_shape_setter_cls, data_parametrized,
+    ):
+        class MyModule(torch.nn.Module):
+            def __init__(self, other_input_dim=22):
+                super().__init__()
+                self.layer = torch.nn.Linear(other_input_dim, 2)
+            def forward(self, X):
+                return self.layer(X)
+
+        net = net_cls(MyModule, max_epochs=2, callbacks=[
+            input_shape_setter_cls(param_name='other_input_dim'),
+        ])
+
+        X, y = data_parametrized
+        n_input = X.shape[1]
+        net.fit(X, y)
+
+        assert net.module_.layer.in_features == n_input
+
+    def test_module_name(
+        self, net_cls, module_cls, input_shape_setter_cls, data_parametrized,
+    ):
+        class MyNet(net_cls):
+            def initialize_module(self):
+                kwargs = self.get_params_for('module')
+                self.module_ = self.module(**kwargs)
+
+                kwargs = self.get_params_for('module2')
+                self.module2_ = self.module(**kwargs)
+
+        net = MyNet(
+            module=module_cls,
+            max_epochs=2,
+            callbacks=[
+                input_shape_setter_cls(module_name='module'),
+                input_shape_setter_cls(module_name='module2'),
+            ],
+        )
+
+        X, y = data_parametrized
+        n_input = X.shape[1]
+        net.fit(X, y)
+
+        assert net.module_.layer.in_features == n_input
+        assert net.module2_.layer.in_features == n_input
+
+    def test_no_module_reinit_when_already_correct(
+        self, net_cls, module_cls, input_shape_setter_cls, data_fixed,
+    ):
+        with patch('skorch.classifier.NeuralNetClassifier.initialize_module',
+                   side_effect=net_cls.initialize_module, autospec=True):
+            net = net_cls(
+                module_cls, max_epochs=2, callbacks=[input_shape_setter_cls()],
+
+                # set the input dim to the correct shape beforehand
+                module__input_dim=data_fixed[0].shape[-1],
+            )
+
+            net.fit(*data_fixed)
+
+            # first initialization due to `initialize()` but not
+            # a second one since the input shape is already correct.
+            assert net.initialize_module.call_count == 1
+
+    def test_no_module_reinit_partial_fit(
+        self, net_cls, module_cls, input_shape_setter_cls, data_fixed,
+    ):
+        with patch('skorch.classifier.NeuralNetClassifier.initialize_module',
+                   side_effect=net_cls.initialize_module, autospec=True):
+            net = net_cls(
+                module_cls, max_epochs=2, callbacks=[input_shape_setter_cls()],
+            )
+
+            net.fit(*data_fixed)
+            # first initialization due to `initialize()`, second
+            # by setting the input dimension in `on_train_begin`
+            assert net.initialize_module.call_count == 2
+
+            net.partial_fit(*data_fixed)
+            # no re-initialization when there was no change in
+            # input dimension.
+            assert net.initialize_module.call_count == 2
