@@ -4,11 +4,11 @@ Should not have any dependency on other skorch packages.
 
 """
 
-from collections.abc import Sequence
+from collections.abc import Mapping, Sequence
 from contextlib import contextmanager
-from distutils.version import LooseVersion
 from enum import Enum
 from functools import partial
+import io
 from itertools import tee
 import pathlib
 import warnings
@@ -16,7 +16,11 @@ import warnings
 import numpy as np
 from scipy import sparse
 import sklearn
+from sklearn.exceptions import NotFittedError
+from sklearn.utils import _safe_indexing as safe_indexing
+from sklearn.utils.validation import check_is_fitted as sk_check_is_fitted
 import torch
+from torch.nn import BCELoss
 from torch.nn import BCEWithLogitsLoss
 from torch.nn import CrossEntropyLoss
 from torch.nn.utils.rnn import PackedSequence
@@ -25,10 +29,11 @@ from torch.utils.data.dataset import Subset
 from skorch.exceptions import DeviceWarning
 from skorch.exceptions import NotInitializedError
 
-if LooseVersion(sklearn.__version__) >= '0.22.0':
-    from sklearn.utils import _safe_indexing as safe_indexing
-else:
-    from sklearn.utils import safe_indexing
+try:
+    import torch_geometric
+    TORCH_GEOMETRIC_INSTALLED = True
+except ImportError:
+    TORCH_GEOMETRIC_INSTALLED = False
 
 
 class Ansi(Enum):
@@ -49,6 +54,11 @@ def is_dataset(x):
     return isinstance(x, torch.utils.data.Dataset)
 
 
+def is_geometric_data_type(x):
+    from torch_geometric.data import Data
+    return isinstance(x, Data)
+
+
 # pylint: disable=not-callable
 def to_tensor(X, device, accept_sparse=False):
     """Turn input data to torch tensor.
@@ -83,7 +93,12 @@ def to_tensor(X, device, accept_sparse=False):
 
     if is_torch_data_type(X):
         return to_device(X, device)
-    if isinstance(X, dict):
+    if TORCH_GEOMETRIC_INSTALLED and is_geometric_data_type(X):
+        return to_device(X, device)
+    if hasattr(X, 'convert_to_tensors'):
+        # huggingface transformers BatchEncoding
+        return X.convert_to_tensors('pt')
+    if isinstance(X, Mapping):
         return {key: to_tensor_(val) for key, val in X.items()}
     if isinstance(X, (list, tuple)):
         return [to_tensor_(x) for x in X]
@@ -103,6 +118,11 @@ def to_tensor(X, device, accept_sparse=False):
     raise TypeError("Cannot convert this data type to a torch tensor.")
 
 
+def _is_slicedataset(X):
+    # Cannot use isinstance because we don't want to depend on helper.py.
+    return hasattr(X, 'dataset') and hasattr(X, 'idx') and hasattr(X, 'indices')
+
+
 def to_numpy(X):
     """Generic function to convert a pytorch tensor to numpy.
 
@@ -116,7 +136,7 @@ def to_numpy(X):
     if isinstance(X, np.ndarray):
         return X
 
-    if isinstance(X, dict):
+    if isinstance(X, Mapping):
         return {key: to_numpy(val) for key, val in X.items()}
 
     if is_pandas_ndframe(X):
@@ -125,12 +145,18 @@ def to_numpy(X):
     if isinstance(X, (tuple, list)):
         return type(X)(to_numpy(x) for x in X)
 
+    if _is_slicedataset(X):
+        return np.asarray(X)
+
     if not is_torch_data_type(X):
         raise TypeError("Cannot convert this data type to a numpy array.")
 
     if X.is_cuda:
         X = X.cpu()
 
+    if hasattr(X, 'is_mps') and X.is_mps:
+        X = X.cpu()
+
     if X.requires_grad:
         X = X.detach()
 
@@ -140,6 +166,9 @@ def to_numpy(X):
 def to_device(X, device):
     """Generic function to modify the device type of the tensor(s) or module.
 
+    PyTorch distribution objects are left untouched, since they don't support an
+    API to move between devices.
+
     Parameters
     ----------
     X : input data
@@ -159,12 +188,17 @@ def to_device(X, device):
     if device is None:
         return X
 
-    if isinstance(X, dict):
-        return {key: to_device(val,device) for key, val in X.items()}
+    if isinstance(X, Mapping):
+        # dict-like but not a dict
+        return type(X)({key: to_device(val, device) for key, val in X.items()})
 
     # PackedSequence class inherits from a namedtuple
     if isinstance(X, (tuple, list)) and (type(X) != PackedSequence):
         return type(X)(to_device(x, device) for x in X)
+
+    if isinstance(X, torch.distributions.distribution.Distribution):
+        return X
+
     return X.to(device)
 
 
@@ -186,7 +220,7 @@ def is_pandas_ndframe(x):
 
 def flatten(arr):
     for item in arr:
-        if isinstance(item, (tuple, list, dict)):
+        if isinstance(item, (tuple, list, Mapping)):
             yield from flatten(item)
         else:
             yield item
@@ -243,7 +277,7 @@ def check_indexing(data):
     if data is None:
         return _indexing_none
 
-    if isinstance(data, dict):
+    if isinstance(data, Mapping):
         # dictionary of containers
         return _indexing_dict
 
@@ -444,14 +478,15 @@ def noop(*args, **kwargs):
 @contextmanager
 def open_file_like(f, mode):
     """Wrapper for opening a file"""
-    new_fd = isinstance(f, (str, pathlib.Path))
-    if new_fd:
-        f = open(f, mode)
+    if isinstance(f, (str, pathlib.Path)):
+        file_like = open(f, mode)
+    else:
+        file_like = f
+
     try:
-        yield f
+        yield file_like
     finally:
-        if new_fd:
-            f.close()
+        file_like.close()
 
 
 # pylint: disable=unused-argument
@@ -531,7 +566,7 @@ def get_map_location(target_device, fallback_device='cpu'):
     return map_location
 
 
-def check_is_fitted(estimator, attributes, msg=None, all_or_any=all):
+def check_is_fitted(estimator, attributes=None, msg=None, all_or_any=all):
     """Checks whether the net is initialized.
 
     Note: This calls ``sklearn.utils.validation.check_is_fitted``
@@ -541,16 +576,15 @@ def check_is_fitted(estimator, attributes, msg=None, all_or_any=all):
     an ``sklearn.exceptions.NotFittedError``.
 
     """
-    if msg is None:
-        msg = ("This %(name)s instance is not initialized yet. Call "
-               "'initialize' or 'fit' with appropriate arguments "
-               "before using this method.")
+    try:
+        sk_check_is_fitted(estimator, attributes, msg=msg, all_or_any=all_or_any)
+    except NotFittedError as exc:
+        if msg is None:
+            msg = ("This %(name)s instance is not initialized yet. Call "
+                   "'initialize' or 'fit' with appropriate arguments "
+                   "before using this method.")
 
-    if not isinstance(attributes, (list, tuple)):
-        attributes = [attributes]
-
-    if not all_or_any([hasattr(estimator, attr) for attr in attributes]):
-        raise NotInitializedError(msg % {'name': type(estimator).__name__})
+        raise NotInitializedError(msg % {'name': type(estimator).__name__}) from exc
 
 
 def _identity(x):
@@ -558,6 +592,18 @@ def _identity(x):
     return x
 
 
+def _make_2d_probs(prob):
+    """Create a 2d probability array from a 1d vector
+
+    This is needed because by convention, even for binary classification
+    problems, sklearn expects 2 probabilities to be returned per row, one for
+    class 0 and one for class 1.
+
+    """
+    y_proba = torch.stack((1 - prob, prob), 1)
+    return y_proba
+
+
 def _sigmoid_then_2d(x):
     """Transform 1-dim logits to valid y_proba
 
@@ -580,11 +626,15 @@ def _sigmoid_then_2d(x):
 
     """
     prob = torch.sigmoid(x)
-    y_proba = torch.stack((1 - prob, prob), 1)
-    return y_proba
+    return _make_2d_probs(prob)
 
 
-def _infer_predict_nonlinearty(net):
+# TODO only needed if multiclass GP classfication is added
+# def _transpose(x):
+    # return x.T
+
+
+def _infer_predict_nonlinearity(net):
     """Infers the correct nonlinearity to apply for this net
 
     The nonlinearity is applied only when calling
@@ -604,8 +654,36 @@ def _infer_predict_nonlinearty(net):
     if isinstance(criterion, BCEWithLogitsLoss):
         return _sigmoid_then_2d
 
+    if isinstance(criterion, BCELoss):
+        return _make_2d_probs
+
     return _identity
 
+    # TODO: Add the code below to _infer_predict_nonlinearity if multiclass GP
+    # classfication is added.
+    # likelihood = getattr(net, 'likelihood_', None)
+    # if likelihood is None:
+    #     return _identity
+    # nonlin = _identity
+    # try:
+    #     import gpytorch
+    #     if isinstance(likelihood, gpytorch.likelihoods.SoftmaxLikelihood):
+    #         # SoftmaxLikelihood returns batch second order
+    #         nonlin = _transpose
+    # except ImportError:
+    #     # there is no gpytorch install
+    #     pass
+    # except AttributeError:
+    #     # gpytorch and pytorch are incompatible
+    #     msg = (
+    #         "Importing gpytorch failed. This is probably because its version is "
+    #         "incompatible with the installed torch version. Please visit "
+    #         "https://github.com/cornellius-gp/gpytorch#installation to check "
+    #         "which versions are compatible"
+    #     )
+    #     warnings.warn(msg)
+    # return nonlin
+
 
 class TeeGenerator:
     """Stores a generator and calls ``tee`` on it to create new generators
